@@ -22,6 +22,7 @@
   @brief Config file parsing
   @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
   @author Copyright (C) 2007 Paul Kube <nodogsplash@kokoro.ucsd.edu>
+  @author Copyright (C) 2015-2022 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
  */
 
 #define _GNU_SOURCE
@@ -106,6 +107,10 @@ typedef enum {
 	oUploadRate,
 	oDownloadBucketRatio,
 	oUploadBucketRatio,
+	oMaxDownloadBucketSize,
+	oMaxUploadBucketSize,
+	oUploadUnrestrictedBursting,
+	oDownloadUnrestrictedBursting,
 	oDownloadQuota,
 	oUploadQuota,
 	oNdsctlSocket,
@@ -180,6 +185,10 @@ static const struct {
 	{ "uploadrate", oUploadRate },
 	{ "download_bucket_ratio", oDownloadBucketRatio },
 	{ "upload_bucket_ratio", oUploadBucketRatio },
+	{ "max_download_bucket_size", oMaxDownloadBucketSize },
+	{ "max_upload_bucket_size", oMaxUploadBucketSize },
+	{ "upload_unrestricted_bursting", oUploadUnrestrictedBursting },
+	{ "download_unrestricted_bursting", oDownloadUnrestrictedBursting },
 	{ "downloadquota", oDownloadQuota },
 	{ "uploadquota", oUploadQuota },
 	{ "syslogfacility", oSyslogFacility },
@@ -289,7 +298,11 @@ config_init(void)
 	config.upload_rate =  DEFAULT_UPLOAD_RATE;
 	config.download_rate = DEFAULT_DOWNLOAD_RATE;
 	config.download_bucket_ratio = DEFAULT_DOWNLOAD_BUCKET_RATIO;
+	config.max_download_bucket_size = DEFAULT_MAX_DOWNLOAD_BUCKET_SIZE;
+	config.download_unrestricted_bursting = DEFAULT_DOWNLOAD_UNRESTRICTED_BURSTING;
 	config.upload_bucket_ratio =  DEFAULT_UPLOAD_BUCKET_RATIO;
+	config.max_upload_bucket_size = DEFAULT_MAX_UPLOAD_BUCKET_SIZE;
+	config.upload_unrestricted_bursting = DEFAULT_UPLOAD_UNRESTRICTED_BURSTING;
 	config.upload_quota =  DEFAULT_UPLOAD_QUOTA;
 	config.download_quota = DEFAULT_DOWNLOAD_QUOTA;
 	config.syslog_facility = DEFAULT_SYSLOG_FACILITY;
@@ -315,12 +328,16 @@ config_init(void)
 	// Set up default FirewallRuleSets, and their empty ruleset policies
 	rs = add_ruleset("trusted-users");
 	rs->emptyrulesetpolicy = safe_strdup(DEFAULT_EMPTY_TRUSTED_USERS_POLICY);
+
 	rs = add_ruleset("trusted-users-to-router");
 	rs->emptyrulesetpolicy = safe_strdup(DEFAULT_EMPTY_TRUSTED_USERS_TO_ROUTER_POLICY);
+
 	rs = add_ruleset("users-to-router");
 	rs->emptyrulesetpolicy = safe_strdup(DEFAULT_EMPTY_USERS_TO_ROUTER_POLICY);
+
 	rs = add_ruleset("authenticated-users");
 	rs->emptyrulesetpolicy = safe_strdup(DEFAULT_EMPTY_AUTHENTICATED_USERS_POLICY);
+
 	rs = add_ruleset("preauthenticated-users");
 	rs->emptyrulesetpolicy = safe_strdup(DEFAULT_EMPTY_PREAUTHENTICATED_USERS_POLICY);
 }
@@ -692,7 +709,6 @@ get_ruleset(const char ruleset[])
 
 	for (tmp = config.rulesets; tmp != NULL
 			&& strcmp(tmp->name, ruleset) != 0; tmp = tmp->next);
-
 	return (tmp);
 }
 
@@ -1091,6 +1107,34 @@ config_read(const char *filename)
 				exit(1);
 			}
 			break;
+		case oMaxDownloadBucketSize:
+			if (sscanf(p1, "%llu", &config.max_download_bucket_size) < 1 || config.max_download_bucket_size < 0) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
+		case oMaxUploadBucketSize:
+			if (sscanf(p1, "%llu", &config.max_upload_bucket_size) < 1 || config.max_upload_bucket_size < 0) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
+		case oDownloadUnrestrictedBursting:
+			if (sscanf(p1, "%d", &config.download_unrestricted_bursting) < 1) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
+		case oUploadUnrestrictedBursting:
+			if (sscanf(p1, "%d", &config.upload_unrestricted_bursting) < 1) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
 		case oDownloadQuota:
 			if (sscanf(p1, "%llu", &config.download_quota) < 1 || config.download_quota < 0) {
 				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
